@@ -32,4 +32,9 @@ val contains : string -> t -> bool
 val pp : Format.formatter -> t -> unit
 (** Pretty print the headers. *)
 
+(**/*)
+
 val parse_ : buf:Buf.t -> IO.Input.t -> t
+val parse_line_ : string -> (string * string, string) result
+
+(**/*)
