@@ -12,20 +12,20 @@ type 'a t = {
 let create ?(clear = ignore) ~mk_item ?(max_size = 512) () : _ t =
   { mk_item; clear; max_size; items = A.make Nil }
 
-let rec acquire_ self =
+let rec acquire self =
   match A.get self.items with
   | Nil -> self.mk_item ()
   | Cons (_, x, tl) as l ->
     if A.compare_and_set self.items l tl then
       x
     else
-      acquire_ self
+      acquire self
 
 let[@inline] size_ = function
   | Cons (sz, _, _) -> sz
   | Nil -> 0
 
-let release_ self x : unit =
+let release self x : unit =
   let rec loop () =
     match A.get self.items with
     | Cons (sz, _, _) when sz >= self.max_size ->
@@ -40,12 +40,17 @@ let release_ self x : unit =
   loop ()
 
 let with_resource (self : _ t) f =
-  let x = acquire_ self in
+  let x = acquire self in
   try
     let res = f x in
-    release_ self x;
+    release self x;
     res
   with e ->
     let bt = Printexc.get_raw_backtrace () in
-    release_ self x;
+    release self x;
     Printexc.raise_with_backtrace e bt
+
+module Raw = struct
+  let release = release
+  let acquire = acquire
+end
