@@ -84,9 +84,11 @@ let unwrap_handler_result req = function
 
 type t = {
   backend: (module IO_BACKEND);
+  enable_logging: bool;
   mutable tcp_server: IO.TCP_server.t option;
   mutable handler: IO.Input.t Request.t -> Response.t;
       (** toplevel handler, if any *)
+  mutable head_middlewares: Head_middleware.t list;
   mutable middlewares: (int * Middleware.t) list;  (** Global middlewares *)
   mutable middlewares_sorted: (int * Middleware.t) list lazy_t;
       (** sorted version of {!middlewares} *)
@@ -127,6 +129,9 @@ let add_middleware ~stage self m =
   self.middlewares <- (stage, m) :: self.middlewares;
   self.middlewares_sorted <- lazy (sort_middlewares_ self.middlewares)
 
+let add_head_middleware (self : t) m : unit =
+  self.head_middlewares <- m :: self.head_middlewares
+
 let add_decode_request_cb self f =
   (* turn it into a middleware *)
   let m h req ~resp =
@@ -250,13 +255,14 @@ let add_route_server_sent_handler ?accept ?(middlewares = []) self route f =
     end in
     (try f req (module SSG : SERVER_SENT_GENERATOR)
      with Exit_SSE -> IO.Output.close oc);
-    Log.info (fun k -> k "closed SSE connection")
+    if self.enable_logging then Log.info (fun k -> k "closed SSE connection")
   in
   add_route_handler_ self ?accept ~meth:`GET route ~tr_req f
 
 let add_upgrade_handler ?(accept = fun _ -> Ok ()) ?(middlewares = [])
     (self : t) route f : unit =
   let ph req : handler_result option =
+    let middlewares = List.rev_append self.head_middlewares middlewares in
     if req.Request.meth <> `GET then
       None
     else (
@@ -272,14 +278,17 @@ let add_upgrade_handler ?(accept = fun _ -> Ok ()) ?(middlewares = [])
 
 let clear_bytes_ bs = Bytes.fill bs 0 (Bytes.length bs) '\x00'
 
-let create_from ?(buf_size = 16 * 1_024) ?(middlewares = []) ~backend () : t =
+let create_from ?(enable_logging = not Log.dummy) ?(buf_size = 16 * 1_024)
+    ?(head_middlewares = []) ?(middlewares = []) ~backend () : t =
   let handler _req = Response.fail ~code:404 "no top handler" in
   let self =
     {
       backend;
+      enable_logging;
       tcp_server = None;
       handler;
       path_handlers = [];
+      head_middlewares;
       middlewares = [];
       middlewares_sorted = lazy [];
       bytes_pool =
@@ -311,11 +320,14 @@ let find_map f l =
   in
   aux f l
 
-let string_as_list_contains_ (s : string) (sub : string) : bool =
+let header_list_contains_ (s : string) (name : string) : bool =
+  let name' = String.lowercase_ascii name in
   let fragments = String.split_on_char ',' s in
-  List.exists (fun fragment -> String.trim fragment = sub) fragments
+  List.exists
+    (fun fragment -> String.lowercase_ascii (String.trim fragment) = name')
+    fragments
 
-(* handle client on [ic] and [oc] *)
+(** handle client on [ic] and [oc] *)
 let client_handle_for (self : t) ~client_addr ic oc : unit =
   Pool.with_resource self.bytes_pool @@ fun bytes_req ->
   Pool.with_resource self.bytes_pool @@ fun bytes_res ->
@@ -323,7 +335,7 @@ let client_handle_for (self : t) ~client_addr ic oc : unit =
 
   (* how to log the response to this query *)
   let log_response (req : _ Request.t) (resp : Response.t) =
-    if not Log.dummy then (
+    if self.enable_logging && not Log.dummy then (
       let msgf k =
         let elapsed = B.get_time_s () -. req.start_time in
         k
@@ -350,14 +362,14 @@ let client_handle_for (self : t) ~client_addr ic oc : unit =
   let handle_exn e bt : unit =
     let msg = Printexc.to_string e in
     let resp = Response.fail ~code:500 "server error: %s" msg in
-    if not Log.dummy then log_exn msg bt;
+    if self.enable_logging && not Log.dummy then log_exn msg bt;
     Response.Private_.output_ ~bytes:bytes_res oc resp
   in
 
   let handle_bad_req req e bt =
     let msg = Printexc.to_string e in
     let resp = Response.fail ~code:500 "server error: %s" msg in
-    if not Log.dummy then (
+    if self.enable_logging && not Log.dummy then (
       log_exn msg bt;
       log_response req resp
     );
@@ -379,7 +391,7 @@ let client_handle_for (self : t) ~client_addr ic oc : unit =
 
       (* check headers *)
       (match Request.get_header req "connection" with
-      | Some str when string_as_list_contains_ str "Upgrade" -> ()
+      | Some str when header_list_contains_ str "Upgrade" -> ()
       | _ -> bad_reqf 426 "connection header must contain 'Upgrade'");
       (match Request.get_header req "upgrade" with
       | Some u when u = UP.name -> ()
@@ -390,7 +402,8 @@ let client_handle_for (self : t) ~client_addr ic oc : unit =
       match UP.handshake client_addr req with
       | Error msg ->
         (* fail the upgrade *)
-        Log.error (fun k -> k "upgrade failed: %s" msg);
+        if self.enable_logging then
+          Log.error (fun k -> k "upgrade failed: %s" msg);
         send_resp @@ Response.make_raw ~code:429 "upgrade required"
       | Ok (headers, handshake_st) ->
         (* send the upgrade reply *)
@@ -412,10 +425,10 @@ let client_handle_for (self : t) ~client_addr ic oc : unit =
 
   (* merge per-request middlewares with the server-global middlewares *)
   let get_middlewares ~handler_middlewares () : _ list =
-    let global_middlewares = Lazy.force self.middlewares_sorted in
-    if handler_middlewares = [] then
+    if handler_middlewares = [] then (
+      let global_middlewares = Lazy.force self.middlewares_sorted in
       global_middlewares
-    else
+    ) else
       sort_middlewares_ (List.rev_append handler_middlewares self.middlewares)
   in
 
